@@ -1,6 +1,6 @@
 # CELERY
 # Default broker URL. See http://celery.readthedocs.org/en/latest/configuration.html#broker-url.
-BROKER_URL = 'amqp://celery:celery@127.0.0.1:5672'
+BROKER_URL = 'redis://:celery@127.0.0.1:6379'
 
 # Disable connection pooling. Connections may be severed by load balancers.
 # This forces the application to connect explicitly to the broker each time
@@ -63,6 +63,31 @@ ECOMMERCE_SERVICE_USERNAME = 'ecommerce_worker'
 # For example, the ECOMMERCE_API_ROOT value is different from one ecommerce site to the next
 SITE_OVERRIDES = None
 
+# .. toggle_name: BRAZE['BRAZE_ENABLE']
+# .. toggle_implementation: PythonConstant
+# .. toggle_default: False
+# .. toggle_description: Toggle for allowing emails to be sent via Braze instead of sailthru
+# .. toggle_use_cases: open_edx
+# .. toggle_creation_date: 2021-02-09
+# .. toggle_target_removal_date: None
+# .. toggle_warnings: None
+# .. toggle_tickets: ENT-4071
+BRAZE = {
+    'BRAZE_ENABLE': False,
+    'BRAZE_REST_API_KEY': None,
+    'BRAZE_WEBAPP_API_KEY': None,
+    'EMAIL_TEMPLATE_ID': '',
+    'REST_API_URL': 'https://rest.iad-06.braze.com',
+    'MESSAGES_SEND_ENDPOINT': '/messages/send',
+    'EMAIL_BOUNCE_ENDPOINT': '/email/hard_bounces',
+    'NEW_ALIAS_ENDPOINT': '/users/alias/new',
+    'USERS_TRACK_ENDPOINT': '/users/track',
+    'FROM_EMAIL': '<edx-for-business-no-reply@info.edx.org>',
+    # Retry settings for Braze celery tasks
+    'BRAZE_RETRY_SECONDS': 3600,
+    'BRAZE_RETRY_ATTEMPTS': 6,
+    }
+
 # Settings for Sailthru email marketing integration
 SAILTHRU = {
     # Set to false to ignore Sailthru events
